package org.softevo.apibirthmark.anttasks;

import java.io.File;
import java.util.Iterator;
import java.util.Vector;

import org.apache.tools.ant.Task;
import org.apache.tools.ant.types.Path;

import org.softevo.apibirthmark.transformation.FileTransformer;
import org.softevo.apibirthmark.constants.Constants;

/**
 * Ant Task to transform observed programs (insert calls to proxy methods)
 *
 * @author schuler
 *
 */
public class ObservedTransformerTask extends Task {

	protected String idMapDestination;

	protected String idMapSource;

	protected String fileSource;

	protected String destinationDir;

	protected String maxKeyFile;

	protected Vector<Path> paths = new Vector<Path>();

	protected boolean overwriteMode = true;

	protected boolean obfuscate = false;

	/* (non-Javadoc)
	 * @see org.apache.tools.ant.Task#execute()
	 */
	public void execute() {
		Iterator<Path> pathIterator = paths.iterator();
		Vector<String> jarVector = new Vector<String>();
		while (pathIterator.hasNext()) {
			Path path = pathIterator.next();
			String[] includedFiles = path.list();
			for (int i = 0; i < includedFiles.length; i++) {
				String filename = includedFiles[i].replace('\\', '/');
				if (filename.endsWith("jar") || filename.endsWith("zip")) {
					jarVector.add(filename);
					log(filename);
				}
			}
		}
		String[] jars = jarVector.toArray(new String[0]);
		try {
			if (jars.length > 0) {
				for (int i = 0; i < jars.length; i++) {
					String jarFullPath = jars[i];
					String jar = jarFullPath.substring(jarFullPath
							.lastIndexOf("/") + 1);
					String jarName = jar.substring(0, jar.lastIndexOf("."));
					String idFile = idMapDestination + "/" + jarName
							+ "_idFile" + ".xml";
					String fileDestination = new File(destinationDir, jar)
							.toString();
					if (overwriteMode || !new File(fileDestination).exists()) {
						// log("FileTransformer.transformObserved(\""
						// + jarFullPath + "\", \"" + fileDestination
						// + "\", \"" + idMapSource + "\", \"" + idFile
						// + "\", \"" + maxKeyFile + ");");
						try {
							transform(jarFullPath, fileDestination,
									idMapSource, idFile, maxKeyFile);
						} catch (Exception e) {
							log("Exception while instrumenting " + jarFullPath);
							e.printStackTrace();
						}
						maxKeyFile = idFile;
					} else {
						log(fileDestination + " already exists");
					}
				}
			} else { // old behaviour
				String fileDestination;
				if (destinationDir.endsWith("jar")
						|| destinationDir.endsWith("zip")) {
					fileDestination = destinationDir;
				} else {
					fileDestination = destinationDir
							+ "/"
							+ fileSource
									.substring(fileSource.lastIndexOf("/") + 1);
				}
				fileDestination.replace('\\', '/');
				transform(fileSource, fileDestination, idMapSource,
						idMapDestination, maxKeyFile);
			}
		} catch (Exception e) {
			e.printStackTrace();
			throw new RuntimeException(e.getMessage());
		}
	}

	/**
	 * Transform given fileSource
	 *
	 * @param fileSource2
	 * @param fileDestination
	 * @param idMapSource2
	 * @param idMapDestination2
	 * @param maxKeyFile2
	 */
	private void transform(String fileSource2, String fileDestination,
			String idMapSource2, String idMapDestination2, String maxKeyFile2) {
		log(fileSource2 + " , " + fileDestination + " , " + idMapSource2
				+ " , " + idMapDestination2 + " , " + maxKeyFile2);
		FileTransformer.transformObserved(fileSource2, fileDestination,
				idMapSource2, idMapDestination2, maxKeyFile2);

	}

	public void setSource(String s) {
		fileSource = s;
	}

	public void setDestination(String s) {
		destinationDir = s;
	}

	public void setIDMapSource(String s) {
		idMapSource = s;
	}

	public void setIDMapDestination(String s) {
		idMapDestination = s;
	}

	public void setMaxKeyFile(String s) {
		maxKeyFile = s;
	}

	public void addPath(Path path) {
		paths.add(path);
	}

	public void setOverwrite(boolean b) {
		overwriteMode = b;
	}

	public void setObservedCalls(String s) {
		System.setProperty(Constants.KEY_OBSERVED_CALLS, s);
	}

}
