package org.softevo.apibirthmark.anttasks;

import java.io.File;
import java.util.Vector;

import org.apache.tools.ant.BuildException;
import org.apache.tools.ant.taskdefs.Java;
import org.apache.tools.ant.types.CommandlineJava;
import org.apache.tools.ant.types.Path;
import org.apache.tools.ant.types.Commandline.Argument;

import org.softevo.apibirthmark.constants.Constants;

/**
 * Ant Task to run instrumented versions of a program and trace its API
 * interaction
 *
 * @author schuler
 *
 */
public class RunTransformedTask extends Java {


	/**
	 * Alternative way to set the JAVA command. This property is used if no
	 * other java command is given.
	 */
	private static final String INSTRUMENTED_JAVA_COMMAND_PROPERTY = "instrumented.java.command";

	/**
	 * Location of the result File
	 */
	private String resultFile;

	/**
	 * If compare is set to true, the same program is executed twice with the
	 * same Input, which produces a second result of which the first can be
	 * compared to.
	 */
	private boolean compare = false;

	/**
	 * Command used to start the JVM
	 */
	private String javaCommand;

	private Vector<Path> bootClasspathPrepend = new Vector<Path>();

	/*
	 * (non-Javadoc)
	 *
	 * @see org.apache.tools.ant.taskdefs.Java#execute()
	 */
	public void execute() throws BuildException {

		CommandlineJava commandLine = getCommandLine();
		Argument bootclassPathAppend = commandLine.createVmArgument();
		StringBuffer bootClasspathPrependBuffer = new StringBuffer(
				"-Xbootclasspath/p");
		for (Path bootPrependEntry : bootClasspathPrepend) {
			bootClasspathPrependBuffer.append(":" + bootPrependEntry);
		}
		bootclassPathAppend.setLine(bootClasspathPrependBuffer.toString());
		Argument resultFileArgument = commandLine.createVmArgument();
		resultFileArgument.setLine("-DresultFile=" + resultFile);
		setFork(true);
		setNewenvironment(true);
		if (javaCommand == null) {
			String newJavaCommand = getProject().getProperty(
					INSTRUMENTED_JAVA_COMMAND_PROPERTY);
			if (newJavaCommand != null) {
				javaCommand = newJavaCommand;
			} else {
				throw new RuntimeException(
						"No Command specified to start the instrumented JVM");
			}
		}
		setJvm(javaCommand);
		log(getCommandLine().toString());
		super.execute();
		if (compare) {
			File resFile = new File(resultFile);
			File compFile = new File(resFile.getParent(),
					Constants.COMPARE_PREFIX + resFile.getName());
			resultFileArgument.setLine("-DresultFile=" + compFile.toString());
			super.execute();
		}
	}

	/**
	 * setter for property resultFile
	 *
	 * @param s
	 */
	public void setResultFile(String s) {
		resultFile = s;
	}

	/**
	 * setter for compare, if compare is true a compare result is produced by
	 * running the same program again and saving the result in a different file.
	 *
	 * @param compare
	 */
	public void setCompare(boolean compare) {
		this.compare = compare;
	}

	/**
	 * setter for javaCommand, this command is used to start the JVM this field
	 * maybe set if not set a default command from a property is used
	 * (which must be set somewhere in the build file)
	 *
	 * @param javaCommand
	 *            command to start the (instrumented) JVM
	 */
	public void setJavaCommand(String javaCommand) {
		this.javaCommand = javaCommand;
	}

	/**
	 * Prepends an entry to the bootclasspath.
	 *
	 * @param bootPrepend
	 *            Entry that is prepended to the bootclasspath.
	 */
	public void addBootprepend(Path bootPrepend) {
		this.bootClasspathPrepend.add(bootPrepend);
	}

}
