package org.softevo.apibirthmark.asmadapters.calls;

import org.objectweb.asm.ClassAdapter;
import org.objectweb.asm.ClassVisitor;
import org.objectweb.asm.MethodVisitor;

import org.softevo.apibirthmark.asmadapters.calls.observed.ObservedCalls;

import org.softevo.apibirthmark.tracing.mapping.NameToIDMap;

/**
 * ASM {@link ClassAdapter} to replace calls to API methods with
 * calls to the corresponding proxy methods.
 *
 * @author schuler
 *
 */
public class InsertProxyCallsClassAdapter extends ClassAdapter {

	private NameToIDMap idMap;

	private String className;

	private ObservedCalls observedCalls;

	/** Create new InsertProxyCallsClassAdapter
	 * @param cv ClassVisitor the calls are forwarded to
	 * @param observedCalls NameToIDMap that contains the classes and methods to observe
	 * @param idMap NameToIDMap that to store the names of the classes that are instrumented
	 */
	public InsertProxyCallsClassAdapter(ClassVisitor cv,
			ObservedCalls observedCalls, NameToIDMap idMap) {
		super(cv);
		this.observedCalls = observedCalls;
		this.idMap = idMap;
		className = "NO CLASS";
	}

	/* (non-Javadoc)
	 * @see org.objectweb.asm.ClassAdapter#visit(int, int, java.lang.String, java.lang.String, java.lang.String, java.lang.String[])
	 */
	public void visit(int version, int access, String name, String signature,
			String superName, String[] interfaces) {
		super.visit(version, access, name, signature, superName, interfaces);
		className = name;
	}

	/* (non-Javadoc)
	 * @see org.objectweb.asm.ClassAdapter#visitMethod(int, java.lang.String, java.lang.String, java.lang.String, java.lang.String[])
	 */
	public MethodVisitor visitMethod(int access, String name, String desc,
			String signature, String[] exceptions) {
		MethodVisitor mv = super.visitMethod(access, name, desc, signature,
				exceptions);
		if (mv != null) {
			if (name.equals("<init>")) {
				return new InsertProxyCallsInConstructorMethodAdapter(mv,
						className, access, name, desc, observedCalls, idMap);
			} else {
				return new InsertProxyCallsMethodAdapter(mv, className, access,
						name, desc, observedCalls, idMap);
			}
		}
		return mv;
	}
}
