package org.softevo.apibirthmark.asmadapters.calls.observed;

import java.util.Arrays;
import java.util.HashSet;
import java.util.Set;


import org.softevo.apibirthmark.constants.Constants;

import org.softevo.apibirthmark.tracing.mapping.NameToIDMap;

/**
 * Class to exclude all method calls that go to methods of classes contained in a set of
 * excluded classes
 *
 * @author schuler
 */
public class ExcludeCalls implements ObservedCalls {

	private NameToIDMap nameToIDMap;

	private Set<String> excludeSet;

	/**
	 * Creates a new instance of Exclude Calls
	 * @param idMap {@link NameToIDMap} that contains the classes and methods that should be observed.
	 */
	public ExcludeCalls(NameToIDMap idMap) {
		nameToIDMap = idMap;
		excludeSet = new HashSet<String>();
		excludeSet.addAll(Arrays.asList(Constants.excludedClasses));
	}

	/* (non-Javadoc)
	 * @see asmadapters.calls.observed.ObservedCalls#isObservedMethod(java.lang.String, java.lang.String, java.lang.String)
	 */
	public boolean isObservedMethod(String owner, String methodName, String desc) {
		if (excludeSet.contains(owner)) {
			System.err.println(methodName + " is excluded because member of  "
					+ owner);
			return false;
		}
		return nameToIDMap.containsMethod(owner, methodName, desc);
	}

	/* (non-Javadoc)
	 * @see asmadapters.calls.observed.ObservedCalls#isObservedConstructor(java.lang.String, java.lang.String)
	 */
	public boolean isObservedConstructor(String owner, String desc) {
		if (excludeSet.contains(owner)) {
			System.err.println("Constructor for " + owner + " is excluded");
			return false;
		}
		return nameToIDMap.containsMethod(owner, "<init>", desc);
	}
}
