package org.softevo.apibirthmark.asmadapters.calls.observed;

import java.io.IOException;

import org.softevo.apibirthmark.tracing.mapping.NameToIDMap;
import org.softevo.apibirthmark.constants.Constants;

/**
 * All calls to methods that are contained in a given {@link NameToIDMap} are
 * instrumented
 *
 * @author schuler
 *
 */
public class ObservedCallsFromMap implements ObservedCalls {

	private NameToIDMap nameToIDMap;

	/**
	 * Created a new instance of ObservedCallsFromMap
	 *
	 * @param idMap
	 *            NameToIDMap that contains the methods and constructors to
	 *            observe
	 */
	public ObservedCallsFromMap(NameToIDMap idMap) {
		nameToIDMap = idMap;
	}

	/*
	 * (non-Javadoc)
	 *
	 * @see asmadapters.calls.observed.ObservedCalls#isObservedMethod(java.lang.String,
	 *      java.lang.String, java.lang.String)
	 */
	public boolean isObservedMethod(String owner, String methodName, String desc) {
		return nameToIDMap.containsMethod(owner, methodName, desc);
	}

	/*
	 * (non-Javadoc)
	 *
	 * @see asmadapters.calls.observed.ObservedCalls#isObservedConstructor(java.lang.String,
	 *      java.lang.String)
	 */
	public boolean isObservedConstructor(String owner, String desc) {
		return nameToIDMap.containsMethod(owner, "<init>", desc);
	}

	/**
	 * For testing purposes
	 *
	 * @param args
	 * @throws IOException
	 */
	public static void main(String[] args) throws IOException {
		ObservedCallsFromMap ob = new ObservedCallsFromMap(NameToIDMap
				.getFromFile(Constants.RUNTIME_IDMAP_LOCATION));
		System.out.println(ob.isObservedMethod("java/lang/Object", null, null));
		System.out.println(ob.isObservedMethod("java/lang/String", null, null));
	}
}
