package org.softevo.apibirthmark.asmadapters.proxies;

import org.objectweb.asm.MethodVisitor;

import org.softevo.apibirthmark.asmadapters.BytecodeTasks;

import org.softevo.apibirthmark.tracing.mapping.NameToIDMap;

/**
 * MethodAdapter used to generate proxy method. Here the code is copied instead of
 * calling the original method, because some methods rely on the depth of the
 * method stack.Furthermore tracing code is inserted
 *
 * @author schuler
 *
 */
public class GenerateCopyProxyMethodAdapter extends ProxyMethodAdapter {

	private String className;

	private String methodName;

	private String methodDescriptor;

	private NameToIDMap idMap;

	/**
	 * Creates new instance of GenerateProxyClassAdapter
	 *
	 * @param mv
	 *            MethodVisitor that represents the original method
	 * @param mvAccess
	 *            Access modifier of this method
	 * @param mvDescriptor
	 *            Method descriptor
	 * @param proxyMethod
	 *            MethodVisitor that represents the proxy method
	 * @param className
	 *            name of the class the method belongs to
	 * @param methodName
	 *            Name of the method
	 * @param idMap
	 *            NameToIDMap that contains the class and method IDs
	 */
	public GenerateCopyProxyMethodAdapter(MethodVisitor mv, int mvAccess,
			String mvDescriptor, MethodVisitor proxyMethod, String className,
			String methodName, NameToIDMap idMap) {
		super(mv, mvAccess, mvDescriptor, proxyMethod, 1);
		this.className = className;
		this.methodName = methodName;
		this.methodDescriptor = mvDescriptor;
		this.idMap = idMap;
	}

	/*
	 * (non-Javadoc)
	 *
	 * @see asmadapters.proxies.ProxyMethodAdapter#visitCode()
	 */
	public void visitCode() {
		super.visitCode();
		BytecodeTasks.insertTracingStatements(proxyMethod, className,
				methodName, methodDescriptor, idMap, numberOfArguments, false);
	}

}
