package org.softevo.apibirthmark.asmadapters.proxies.classHierarchy;

import java.util.Collection;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.Vector;

import org.softevo.apibirthmark.tracing.mapping.NameToIDMap;

/**
 * Stores the class hierarchy for a set of classes. It is used to find a
 * superclass of the API if a program subclasses any (other than
 * java.lang.object).
 *
 * @author schuler
 *
 */
public class ClassHierarchy {
	private Hashtable<String, ClassEntry> classes = new Hashtable<String, ClassEntry>();

	/**
	 * Add a class entry to this class
	 *
	 * @param classEntry
	 */
	public void addClassEntry(ClassEntry classEntry) {
		classes.put(classEntry.className, classEntry);
	}

	private ClassEntry getClassEntry(String clazz) {
		return (ClassEntry) classes.get(clazz);
	}

	/**
	 * Get classes or interfaces that are contained in relevant Map and where
	 * given class inherits from.
	 *
	 * @param className
	 *            Name of the class
	 *
	 * @param relevantMap
	 *            Map of relevant classes
	 * @return array that contains all relevant classes
	 */
	public String[] getRelevantClasses(String className, NameToIDMap relevantMap) {
		Vector<String> relevantClasses = new Vector<String>();
		ClassEntry[] superClasses = getAllSuperClasses(className);
		boolean relevantSuperClassFound = false;
		Vector<String> processedInterfaces = new Vector<String>();
		for (int i = 0; i < superClasses.length; i++) {
			if (!relevantSuperClassFound
					&& relevantMap.contains(superClasses[i].className)) {
				relevantClasses.add(superClasses[i].className);
				relevantSuperClassFound = true;
			}
			if (superClasses[i].interfaces != null) {
				addAdditionalInterfaces(superClasses[i].interfaces,
						processedInterfaces);
			}
		}
		if (!relevantSuperClassFound) {
			if (superClasses != null) {
				if (superClasses[superClasses.length - 1].superClass != null) {
					relevantClasses
							.add(superClasses[superClasses.length - 1].superClass);
				} else {
					relevantClasses
							.add(superClasses[superClasses.length - 1].className);
				}
			}
		}
		Iterator iter = processedInterfaces.iterator();
		while (iter.hasNext()) {
			String interfaceName = (String) iter.next();
			if (relevantMap.contains(interfaceName)) {
				relevantClasses.add(interfaceName);
			}
		}
		return (String[]) relevantClasses.toArray(new String[0]);
	}

	private void addAdditionalInterfaces(String[] interfaces,
			Vector<String> processedInterfaces) {
		Vector<String> interfaceVector = new Vector<String>();
		for (int j = 0; j < interfaces.length; j++) {
			String interfaceName = interfaces[j];
			if (!processedInterfaces.contains(interfaceName)) {
				interfaceVector.add(interfaceName);
			}
		}
		while (interfaceVector.size() != 0) {// search recursively for new
			// interfaces
			String interfaceName = (String) interfaceVector.remove(0);
			if (!processedInterfaces.contains(interfaceName)) {// if new
				// interface is found
				processedInterfaces.add(interfaceName);
				ClassEntry interfaceClassEntry = getClassEntry(interfaceName);
				if (interfaceClassEntry != null) {
					for (int j = 0; j < interfaceClassEntry.interfaces.length; j++) {
						String superInterface = interfaceClassEntry.interfaces[j];
						interfaceVector.add(superInterface);
					}
				}
			}
		}
	}

	private ClassEntry[] getAllSuperClasses(String className) {
		Vector<ClassEntry> classEntries = new Vector<ClassEntry>();
		ClassEntry classEntry;
		while ((className != null)
				&& (classEntry = getClassEntry(className)) != null) {
			classEntries.add(classEntry);
			className = classEntry.superClass;
		}
		return (ClassEntry[]) classEntries.toArray(new ClassEntry[0]);
	}

	/*
	 * (non-Javadoc)
	 *
	 * @see java.lang.Object#toString()
	 */
	public String toString() {
		StringBuffer buffer = new StringBuffer();
		Collection c = classes.values();
		Iterator iter = c.iterator();
		while (iter.hasNext()) {
			ClassEntry element = (ClassEntry) iter.next();
			buffer.append(element.toString());
		}
		return buffer.toString();
	}

}
