package org.softevo.apibirthmark.asmadapters.proxies.classHierarchy;

import org.objectweb.asm.ClassAdapter;
import org.objectweb.asm.ClassVisitor;

/**
 * ASM ClassAdapter that just processes a class ( no changes are made). It adds
 * a class to given class hierarchy
 *
 * @author schuler
 *
 */
public class HierarchyClassAdapter extends ClassAdapter {

	private ClassHierarchy classHierarchy;

	/**Creates a new Instance of HierarchyClassAdapter
	 * @param cv ClassVisitor to delegate calls
	 * @param classHierarchy classHierarchy the class is added to
	 */
	public HierarchyClassAdapter(ClassVisitor cv, ClassHierarchy classHierarchy) {
		super(cv);
		this.classHierarchy = classHierarchy;
	}

	/* (non-Javadoc)
	 * @see org.objectweb.asm.ClassAdapter#visit(int, int, java.lang.String, java.lang.String, java.lang.String, java.lang.String[])
	 */
	public void visit(int version, int access, String name, String signature,
			String superName, String[] interfaces) {
		super.visit(version, access, name, signature, superName, interfaces);
		classHierarchy
				.addClassEntry(new ClassEntry(name, superName, interfaces));
	}

	/**
	 * Getter for field classHierarchy
	 * @return The classHierarchy
	 */
	public ClassHierarchy getClassHierarchy() {
		return classHierarchy;
	}
}
