package org.softevo.apibirthmark.asmadapters.proxies.observed;

import org.objectweb.asm.ClassVisitor;
import org.objectweb.asm.MethodVisitor;
import org.objectweb.asm.util.CheckMethodAdapter;

import org.softevo.apibirthmark.asmadapters.proxies.GenerateProxyClassAdapter;
import org.softevo.apibirthmark.asmadapters.proxies.classHierarchy.ClassHierarchy;

import org.softevo.apibirthmark.tracing.mapping.NameToIDMap;

/**
 * ASM ClassAdapter that inserts proxy methods in program classes. If a class
 * subclasses API classes, the ID of the API class and methods is used instead
 * of its own ID.
 *
 * @author schuler
 *
 */
public class ObservedProxyClassAdapter extends GenerateProxyClassAdapter {

	private NameToIDMap relevantIDMap;

	protected ClassHierarchy classHierarchy;

	/**
	 * Created new instance of ObservedProxyClassAdapter
	 *
	 * @param cv ClassVisitor calls are forwarded to
	 * @param idMap
	 *            Map that contains class and method IDs
	 * @param relevantIDMap
	 *            This Map contains IDs for the relevant (API) classes
	 * @param classHierarchy
	 *            Class hierarchy of the classes to be instrumented.
	 */
	public ObservedProxyClassAdapter(ClassVisitor cv, NameToIDMap idMap,
			NameToIDMap relevantIDMap, ClassHierarchy classHierarchy) {
		super(cv, idMap);
		this.relevantIDMap = relevantIDMap;
		this.classHierarchy = classHierarchy;
	}

	/* (non-Javadoc)
	 * @see asmadapters.proxies.GenerateProxyClassAdapter#getDefaultAdapter(org.objectweb.asm.MethodVisitor, org.objectweb.asm.MethodVisitor, java.lang.String, int, java.lang.String, java.lang.String, java.lang.String, java.lang.String[], tracing.mapping.NameToIDMap)
	 */
	@Override
	protected MethodVisitor getDefaultAdapter(MethodVisitor mv,
			MethodVisitor proxyMethod, String className2, int access,
			String name, String desc, String signature, String[] exceptions,
			NameToIDMap idMap2) {
		return new CheckMethodAdapter(new ObservedProxyMethodAdapter(mv,
				proxyMethod, className2, access, name, desc, signature,
				exceptions, idMap2, relevantIDMap, classHierarchy));
	}

}
