package org.softevo.apibirthmark.asmadapters.proxies.observed;

import org.apache.log4j.Level;
import org.apache.log4j.Logger;
import org.objectweb.asm.MethodVisitor;

import org.softevo.apibirthmark.asmadapters.proxies.GenerateProxyMethodAdapter;
import org.softevo.apibirthmark.asmadapters.proxies.classHierarchy.ClassHierarchy;

import org.softevo.apibirthmark.tracing.mapping.NameToIDMap;

/**
 * ASM ClassAdapter that inserts proxy methods in program classes. If a class
 * subclasses API classes, the ID of the API class and methods is used instead
 * of its own ID.
 *
 * @author schuler
 *
 */
public class ObservedProxyMethodAdapter extends GenerateProxyMethodAdapter {

	private static Logger logger = Logger
			.getLogger(ObservedProxyMethodAdapter.class);

	static {
		logger.setLevel(Level.ERROR);
	}

	/**
	 * Generates the proxy method of this method and evantually replaces IDs if
	 * an API class is subclassed.
	 *
	 * @param mv
	 *            ASM method Visitor
	 * @param proxyMethod
	 *            The new generated proxy method
	 * @param className
	 *            Name of the class that is instrumented
	 * @param access
	 *            Access modifiers of this method
	 * @param methodName
	 *            Name of the method
	 * @param desc
	 *            Method Descriptor
	 * @param signature
	 *            Signature of the method
	 * @param exceptions
	 *            Exceptions thrown by this method
	 * @param idMap
	 *            NameToIDMap that contains the class and methodIDs
	 * @param relevantMap
	 *            NameToIDMap that contains the IDs of the relevant (API)
	 *            classes
	 * @param classHierarchy
	 *            Class hierarchy to find super classes contained in the set of
	 *            relevant classes
	 */
	public ObservedProxyMethodAdapter(MethodVisitor mv,
			MethodVisitor proxyMethod, String className, int access,
			String methodName, String desc, String signature,
			String[] exceptions, NameToIDMap idMap, NameToIDMap relevantMap,
			ClassHierarchy classHierarchy) {
		super(mv, proxyMethod, className, access, methodName, desc, signature,
				exceptions, idMap);
		if (!relevantMap.contains(className)) {
			logger.info("Irrelevant Class: " + className);
			String[] relevantClasses = classHierarchy.getRelevantClasses(
					className, relevantMap);
			for (int i = 0; i < relevantClasses.length; i++) {
				if (relevantMap.containsMethod(relevantClasses[i], methodName,
						desc)) {
					this.logClassName = relevantClasses[i];
					logger.info("ReplacingIDs for Class: " + className + " ("
							+ idMap.getClassID(className) + ") with: "
							+ this.logClassName + " ("
							+ relevantMap.getClassID(this.logClassName) + ") ");
					idMap.insertNameToIDEntry(this.logClassName, relevantMap
							.getNameToIDEntry(this.logClassName));
					break;
				}
			}
		}
	}
}
