package org.softevo.apibirthmark.tracing.mapping;

import java.io.IOException;

import org.apache.log4j.Logger;


/**
 * Class to read and combine IDMaps from files
 * @author schuler
 *
 */
public class CombineFiles {

	private static Logger logger = Logger.getLogger(CombineFiles.class);

	/**
	 * IDMaps are read from given filenames and then combined in target
	 * destination.
	 *
	 * @param filenames
	 * @param destination
	 * @throws IOException
	 */
	public static void combineIDMaps(String[] filenames, String destination)
			throws IOException {
		NameToIDMap[] maps = new NameToIDMap[filenames.length];
		for (int i = 0; i < filenames.length; i++) {
			maps[i] = NameToIDMap.getFromFile(filenames[i]);
		}
		NameToIDMap combinedMap = maps[0];
		for (int i = 1; i < maps.length; i++) {
			combinedMap = NameToIDMap.combine(combinedMap, maps[i]);
		}
		logger.info("Writing " + destination);
		combinedMap.saveToFile(destination);
	}

	/**
	 * Combine IDMaps.
	 * Usage:  CombineFiles file1 file2 ... fileX destinationFile
	 * @param args
	 * @throws IOException
	 */
	public static void main(String[] args) throws IOException {
		if (args.length > 2) {
			String[] filenames = new String[args.length - 1];
			for (int i = 0; i < filenames.length; i++) {
				filenames[i] = args[i];
			}
			combineIDMaps(filenames, args[args.length - 1]);
		}
	}
}
