package org.softevo.apibirthmark.tracing.mapping;

import java.util.HashMap;
import java.util.Map;

/**
 * IDToNameEntry stores the mapping of IDs to names for one class.
 *
 * @author schuler
 *
 */
public class IDToNameEntry {

	String className;

	HashMap<Integer, String> methodHashMap;

	/**
	 * Created a IDToNameEntry
	 *
	 * @param className
	 *            name of the class
	 * @param hashMap
	 *            map to store the mapping of IDs to names
	 */
	public IDToNameEntry(String className, HashMap<Integer, String> hashMap) {
		this.className = className;
		methodHashMap = hashMap;
	}

	/**
	 * Return the name of the method with the specified ID
	 *
	 * @param methodID
	 *            ID of the method
	 * @return
	 */
	public String getMethodName(int methodID) {
		if (methodHashMap.containsKey(methodID)) {
			return methodHashMap.get(methodID);
		}
		throw new IDMapError("Method ID not contained in hashmap\nmethodID: "
				+ methodID);
	}

	/* (non-Javadoc)
	 * @see java.lang.Object#equals(java.lang.Object)
	 */
	@Override
	public boolean equals(Object obj) {
		return className.equals(((IDToNameEntry) obj).className);
	}

	/* (non-Javadoc)
	 * @see java.lang.Object#hashCode()
	 */
	@Override
	public int hashCode() {
		return className.hashCode();
	}

	/* (non-Javadoc)
	 * @see java.lang.Object#toString()
	 */
	@Override
	public String toString() {
		StringBuilder buffer = new StringBuilder();
		buffer.append("Class name: ").append(className).append('\n');
		for (Map.Entry element : methodHashMap.entrySet()) {
			buffer.append(element.getKey()).append(" : ").append(
					element.getValue());
		}
		return buffer.toString();
	}
}
