package org.softevo.apibirthmark.tracing.mapping;

import java.util.HashMap;
import java.util.Iterator;
import java.util.Set;

/**
 * NameToIDEntry stores the mapping of method names to IDs for one class.
 *
 * @author schuler
 *
 */
public class NameToIDEntry {

	private int classID;

	private HashMap<String, Integer> methodToIDHashMap = new HashMap<String, Integer>();

	private int methodMaxKey;

	private static final int METHODFACTOR = 10000;

	/**
	 * Creates a new NameToIDEntry for with class ID.
	 *
	 * @param classID
	 *            ID of the class
	 */
	public NameToIDEntry(int classID) {
		this.classID = classID;
		methodMaxKey = classID * NameToIDEntry.METHODFACTOR;
	}

	/**
	 * Getter for class ID.
	 *
	 * @return The class ID
	 */
	public int getClassID() {
		return classID;
	}

	/**
	 * Insert the method into this NameToIDEntry
	 *
	 * @param methodName
	 *            Name of the method
	 * @param methodDescription
	 *            Method Descriptor
	 */
	public void insertMethod(String methodName, String methodDescription) {
		if (methodToIDHashMap.containsKey(methodName + methodDescription)) {
			throw new RuntimeException(
					"Method already contained in HashMap\n\nMethodn name: "
							+ methodName + " " + methodDescription);
		}
		methodToIDHashMap.put(methodName + methodDescription, new Integer(
				methodMaxKey));
		methodMaxKey++;
	}

	/**
	 * Return the ID for given method
	 *
	 * @param methodName
	 *            Name of the method
	 * @param methodDescription
	 *            Method Descriptor
	 * @return The ID of the method
	 */
	public int getMethodID(String methodName, String methodDescription) {
		String key = methodName + methodDescription;
		if (methodToIDHashMap.containsKey(key)) {
			return ((Integer) methodToIDHashMap.get(key)).intValue();
		}
		throw new RuntimeException(
				"Method not contained in HashMap\nMethod name: " + key);
	}

	/**
	 * Returns a reversed HashMap with keys as values and vice versa.
	 *
	 * @return Returns a reversed HashMap with keys as values and vice versa.
	 *
	 */
	public HashMap<Integer, String> getIDtoNameHashMap() {
		Set keySet = methodToIDHashMap.keySet();
		HashMap<Integer, String> idtoNameHashMap = new HashMap<Integer, String>();
		Iterator it = keySet.iterator();
		while (it.hasNext()) {
			String methodName = (String) it.next();
			Integer id = (Integer) methodToIDHashMap.get(methodName);
			idtoNameHashMap.put(id, methodName);
		}
		return idtoNameHashMap;
	}

	/**
	 * Checks if this method is already contained in this NameToIDEntry.
	 *
	 * @param name
	 *            Name of the method
	 * @param desc
	 *            Method descriptor
	 * @return
	 */
	public boolean containsMethod(String name, String desc) {
		return methodToIDHashMap.containsKey(name + desc);
	}

	/**
	 * Return the class ID for given method ID. The class ID is encoded in the
	 * method ID
	 *
	 * @param methodID
	 *            ID of the method
	 * @return the class ID
	 */
	public static int getClassFromMethodID(int methodID) {
		return methodID / NameToIDEntry.METHODFACTOR;
	}

}
