package org.softevo.apibirthmark.tracing.result.birthmark.callee;

import java.util.Formatter;

import org.softevo.apibirthmark.tracing.result.AbstractTraceResult;
import org.softevo.apibirthmark.tracing.result.birthmark.Birthmark;

/**
 * CalleeSequenceBirthmark computes a birthmark based on callee sequences. If a
 * callee sequences is contained in both results then this sequence supports the
 * Birthmark. The number of these sequence is divided by the total number of
 * callee sequences in both results.
 *
 * @author schuler
 *
 */
public class CalleeSequenceBirthmark extends Birthmark {

	protected AbstractTraceResult firstIntersectedResult;

	protected AbstractTraceResult compareIntersectedResult;

	public CalleeSequenceBirthmark(AbstractTraceResult firstResult,
			AbstractTraceResult compareResult) {
		this.firstResult = firstResult;
		this.compareResult = compareResult;
	}

	public float getBirthmarkValue() {
		int sequences = firstIntersectedResult.getNumberOfCalleeSequences()
				+ compareIntersectedResult.getNumberOfCalleeSequences();
		if (firstIntersectedResult.getNumberOfCalleeSequences() != compareIntersectedResult
				.getNumberOfCalleeSequences()) {
			throw new RuntimeException("intersecting sequences unequal");
		}
		int divisor = firstResult.getNumberOfCalleeSequences()
				+ compareResult.getNumberOfCalleeSequences();
		return sequences / (float) divisor;
	}

	public void computeBirthmark() {
		firstIntersectedResult = AbstractTraceResult.intersectByCalleeID(
				firstResult, compareResult);
		if (firstIntersectedResult.getNumberOfCalleeSequences() != AbstractTraceResult
				.intersectByCalleeID(compareResult, firstResult)
				.getNumberOfCalleeSequences()) {
			throw new RuntimeException(
					"intersecting sequences unequal (not cumtative)");
		}
		compareIntersectedResult = (AbstractTraceResult) AbstractTraceResult.intersectByCalleeID(
				compareResult, firstResult);
	}

	@Override
	public String toString() {
		Formatter floatFormater = new Formatter(new StringBuilder());
		floatFormater.format(FLOAT_FORMAT, getBirthmarkValue());
		return floatFormater.toString();
	}

}
