package org.softevo.apibirthmark.tracing.result.birthmark.callee;

import org.softevo.apibirthmark.tracing.result.AbstractTraceResult;
import org.softevo.apibirthmark.tracing.result.birthmark.ContainedBirthmark;

/**
 *
 * ContainedCalleeSequenceBirthmark computes how much of one result is contained
 * in the other and takes the maximum of these two values as a birthmark. The
 * value is computed by intersecting the calleeSequenceSets and dividing it by
 * the number of total calleeSequneces for each result.
 *
 *
 * @author schuler
 *
 */
public class ContainedCalleeSequenceBirthmark extends ContainedBirthmark {

	protected AbstractTraceResult firstIntersectedResult;

	protected AbstractTraceResult compareIntersectedResult;

	public ContainedCalleeSequenceBirthmark(AbstractTraceResult firstResult,
			AbstractTraceResult compareResult) {
		this.firstResult = firstResult;
		this.compareResult = compareResult;
	}

	@Override
	protected float getFirstDivisor() {
		return firstResult.getNumberOfCalleeSequences();
	}

	@Override
	protected float getCompareDivisor() {
		return compareResult.getNumberOfCalleeSequences();
	}

	@Override
	protected float getCompareSupportingSequences() {
		return compareIntersectedResult.getNumberOfCalleeSequences();
	}

	@Override
	protected float getFirstSupportingSequences() {
		return firstIntersectedResult.getNumberOfCalleeSequences();
	}

	@Override
	public void computeBirthmark() {
		firstIntersectedResult = AbstractTraceResult.intersectByCalleeID(
				firstResult, compareResult);
		compareIntersectedResult = AbstractTraceResult.intersectByCalleeID(
				compareResult, firstResult);
	}
}
