package org.softevo.apibirthmark.tracing.result.process;

import java.io.File;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;
import java.util.Vector;

import org.softevo.apibirthmark.tracing.result.AbstractTraceResult;
import org.softevo.apibirthmark.tracing.result.TraceObjectResult;
import org.softevo.apibirthmark.util.ObjectIO;
import org.softevo.apibirthmark.constants.Constants;
import org.softevo.apibirthmark.constants.Constants.ImageReaders;

/**
 * Class that provide sets of result usually in a Map where the key is the name
 * of the program.
 *
 * @author schuler
 *
 */
public class ResultSets {

	private static Set<String> imageReaderSet;
	static {
		imageReaderSet = new HashSet<String>();
		for (ImageReaders imageReaders : ImageReaders.values()) {
			imageReaderSet.add(imageReaders.toString());
		}
	}

	/**
	 * Return all png results includes compare results
	 *
	 * @return
	 */
	public static Map<String, AbstractTraceResult> getAllPngResults() {
		Map<String, AbstractTraceResult> map = ResultFromFiles
				.getResultsFromDirectory(new File(Constants.RESULT_DIRECTORY
						+ "/png/"));
		return map;
	}

	/**
	 * Return all png results excludes compare results
	 *
	 * @return
	 */
	public static Map<String, AbstractTraceResult> getPngResults() {
		return filterCompare(getAllPngResults());
	}

	/**
	 * Return all png reader results includes compare results
	 *
	 * @return
	 */
	public static Map<String, AbstractTraceResult> getAllPngReaderResults() {
		Map<String, AbstractTraceResult> map = ResultFromFiles
				.getResultsFromDirectory(new File(Constants.PNG_RESULT_DIR));
		Map<String, AbstractTraceResult> resultMap = new HashMap<String, AbstractTraceResult>();
		for (Map.Entry<String, AbstractTraceResult> entry : map.entrySet()) {
			if (containsImageReader(entry.getKey())) {
				resultMap.put(entry.getKey(), entry.getValue());
			}
		}
		return resultMap;
	}

	private static boolean containsImageReader(String key) {
		for (ImageReaders imageReaders : ImageReaders.values()) {
			if (key.toLowerCase().contains(imageReaders.toString().toLowerCase())) {
				return true;
			}
		}
		return false;
	}

	/**
	 * Return all png reader results excludes compare results
	 *
	 * @return
	 */
	public static Map<String, AbstractTraceResult> getPngReaderResults() {
		return filterCompare(getAllPngReaderResults());
	}

	/**
	 * Return all XML results includes compare results
	 *
	 * @return
	 */
	public static Map<String, AbstractTraceResult> getAllXMLResults() {
		Map<String, AbstractTraceResult> map = ResultFromFiles
				.getResultsFromDirectory(new File(Constants.RESULT_DIRECTORY
						+ "/xml/"));
		return map;
	}

	/**
	 * Return all SaxTest results excludes compare results
	 *
	 * @return
	 */
	public static Map<String, AbstractTraceResult> getSaxTestResults() {
		return filterCompare(getAllSaxTestResults());
	}

	/**
	 * Return all SaxTest results includes compare results
	 *
	 * @return
	 */
	public static Map<String, AbstractTraceResult> getAllSaxTestResults() {
		Map<String, AbstractTraceResult> map = ResultFromFiles
				.getResultsFromDirectory(new File(Constants.RESULT_DIRECTORY
						+ "/xml/saxtest"));
		return map;
	}

	/**
	 * Return all SAXBench results
	 *
	 * @return
	 */
	public static Map<String, AbstractTraceResult> getAllSaxBenchResults() {
		return ResultFromFiles.getResultsFromDirectory(new File(
				Constants.RESULT_DIRECTORY + "/xml/saxbench"));
	}

	/**
	 * Return SAXBench results minus compare results
	 *
	 * @return
	 */
	public static Map<String, AbstractTraceResult> getSaxBenchResults() {
		return filterCompare(getAllSaxBenchResults());
	}

	/**
	 * Return Combined SAXBench results
	 *
	 * @return
	 */
	public static Map<String, AbstractTraceResult> getAllCombinedSaxBenchResults() {
		Map<String, AbstractTraceResult> map = ResultFromFiles
				.getResultsFromDirectory(new File(Constants.RESULT_DIRECTORY
						+ "/xml/saxbench"));
		return combineMultipleResult(map);
	}

	/**
	 * Return Obfuscated Combined SAXBench results
	 *
	 * @return
	 */
	public static Map<String, AbstractTraceResult> getCombinedObfuscatedSaxBenchResults() {
		Map<String, AbstractTraceResult> map = ResultFromFiles
				.getResultsFromDirectory(new File(Constants.RESULT_DIRECTORY
						+ "/xml/obfuscatedSaxBench"));
		return combineMultipleResult(map);
	}

	/**
	 * Return Combined SAXBench results minus compare results
	 *
	 * @return
	 */
	public static Map<String, AbstractTraceResult> getCombinedSaxBenchResults() {
		return filterCompare(getAllCombinedSaxBenchResults());
	}


	/**
	 * Return all XML results excludes compare results
	 *
	 * @return
	 */
	public static Map<String, AbstractTraceResult> getXMLResults() {
		return filterCompare(getAllXMLResults());
	}

	/**
	 * Takes a map of results and filters the compare results. Returns a new map
	 * without these results
	 *
	 * @param map
	 * @return
	 */
	private static Map<String, AbstractTraceResult> filterCompare(
			Map<String, AbstractTraceResult> map) {
		Map<String, AbstractTraceResult> resultMap = new HashMap<String, AbstractTraceResult>();
		for (Map.Entry<String, AbstractTraceResult> entry : map.entrySet()) {
			if (!entry.getKey().startsWith(Constants.COMPARE_PREFIX)) {
				resultMap.put(entry.getKey(), entry.getValue());
			}
		}
		return resultMap;
	}

	public static void main(String[] args) {
		Map<String, AbstractTraceResult> map = getAllPngResults();

		for (Map.Entry<String, AbstractTraceResult> entry : map.entrySet()) {
			System.out.println(entry.getKey());
		}
		System.out.println("\n\n\n");
		map = getPngResults();
		for (Map.Entry<String, AbstractTraceResult> entry : map.entrySet()) {
			System.out.println(entry.getKey());
		}
	}

	public static Map<String, AbstractTraceResult> getZKMResults() {
		Map<String, AbstractTraceResult> map = ResultFromFiles
				.getResultsFromDirectory(new File(Constants.RESULT_DIRECTORY
						+ "/obfuscated/zkm/"));
		return map;
	}

	public static Map<String, AbstractTraceResult> getZKMExcludeResults() {
		Map<String, AbstractTraceResult> map = ResultFromFiles
				.getResultsFromDirectory(new File(Constants.RESULT_DIRECTORY
						+ "/obfuscated/zkmExclude/"));
		return map;
	}

	public static Map<String, AbstractTraceResult> getPngReaderExcludeResults() {
		Map<String, AbstractTraceResult> map = ResultFromFiles
				.getResultsFromDirectory(new File(Constants.RESULT_DIRECTORY
						+ "/pngExclude"));
		return map;

	}

	public static Map<String, AbstractTraceResult> getAllCombinedPngReaderResults() {
		Map<String, AbstractTraceResult> map = getAllPngReaderResults();
		return combineMultipleResult(map);
	}

	public static Map<String, AbstractTraceResult> getCombinedPngReaderResults() {
		return filterCompare(getAllCombinedPngReaderResults());
	}

	/**
	 * Combine results in the given map. Result files that have the same prefix
	 * and end with a number are combined.
	 *
	 * @param map
	 * @return
	 */
	private static Map<String, AbstractTraceResult> combineMultipleResult(
			Map<String, AbstractTraceResult> map) {
		HashMap<String, Vector<String>> combinekeys = new HashMap<String, Vector<String>>();
		for (String key : map.keySet()) {
			if (key.matches(".+[_]\\d+")) {
				String newKey = key.substring(0, key.lastIndexOf('_'));
				if (!combinekeys.containsKey(newKey)) {
					combinekeys.put(newKey, new Vector<String>());
				}
				combinekeys.get(newKey).add(key);
			}
		}
		// Map<String, TraceObjectResult> combinedResults = new HashMap<String,
		// TraceObjectResult>();
		for (String newkey : combinekeys.keySet()) {
			AbstractTraceResult newResult = new TraceObjectResult();
			System.out.println("combining" + newkey);
			for (String key : combinekeys.get(newkey)) {
				if (newResult.getNumberOfSequences() > 0) {
					newResult.intersectByCalleeID(map.get(key));
				} else {
					newResult.add(map.get(key));
				}
				System.out.println("\t" + key);
				map.remove(key);
			}
			map.put(newkey, newResult);
		}
		return map;
	}


	/**
	 * Save results of map to directory
	 *
	 * @param resultMap
	 *            Map that contains the results
	 * @param dir
	 *            directory to save the results
	 */
	public static void saveResultsToDir(
			Map<String, AbstractTraceResult> resultMap, File dir) {
		for (Map.Entry<String, AbstractTraceResult> entry : resultMap.entrySet()) {
			ObjectIO.saveToFile(new File(dir, entry.getKey() + ".ser"), entry
					.getValue());
		}
	}

}
