package org.softevo.apibirthmark.tracing.result.process.experiments;

import java.io.FileNotFoundException;
import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Map;

import org.softevo.apibirthmark.constants.Constants;

import org.softevo.apibirthmark.tracing.mapping.IDToNameMap;
import org.softevo.apibirthmark.tracing.mapping.NameToIDMap;
import org.softevo.apibirthmark.tracing.result.AbstractTraceResult;
import org.softevo.apibirthmark.tracing.result.birthmark.callee.CalleeSequenceBirthmark;
import org.softevo.apibirthmark.tracing.result.process.ComputeBirthmarks;
import org.softevo.apibirthmark.tracing.result.process.ResultSets;

/**
 * Class that processes the tracing results and computes the pairwise
 * credibility for all results
 *
 * @author schuler
 *
 */
public class Credibility {

	private static boolean printSequenceSets = false;

	private static boolean showNumberOfSequences = true;

	private static boolean writeToFile = false;

	private static boolean latexOutput = false;

	private static Map<String, Float> checkCredibility(
			Map<String, AbstractTraceResult> resultMap, String resultName,
			AbstractTraceResult result) {
		System.out.println("Credibility For " + resultName);
		return ComputeBirthmarks.computeBirthmark(new CalleeSequenceBirthmark(
				null, null), resultMap, result);

	}

	/**
	 * Computes the credibility between a set of results.
	 *
	 * @param results
	 *            Results for which the credibility should be computed
	 * @param compareResults
	 *            Map that contains the results to compare to
	 */
	private static void printCredibilityTable(
			Map<String, AbstractTraceResult> results,
			Map<String, AbstractTraceResult> compareResults) {
		ResultTable resultTable = new ResultTable();
		List<String> orderList = new ArrayList<String>(results.keySet());
		Collections.sort(orderList);
		for (String key : orderList) {
			String compareKey = (Constants.COMPARE_PREFIX + key);
			if (compareResults.containsKey(compareKey)) {
				resultTable.putRow(key, checkCredibility(results, key,
						compareResults.get(compareKey)));
			} else {
				System.out.println("No compare Results");
			}
		}
		AbstractTraceResult[] resultArray = results.values().toArray(
				new AbstractTraceResult[results.size()]);
		System.out.println("Window Size:" + resultArray[0].getWindowSize());
		System.out.println(resultTable);
		if (latexOutput){
		System.out.println(resultTable.toLatexString());
}
		if (printSequenceSets) {
			printSequenceSets(orderList, results);
		}
		if (showNumberOfSequences) {
			showNumberOfSequences(orderList, results);
		}
		if (writeToFile) {
			writeToFile(resultTable);
		}
	}

	private static void printSequenceSets(List<String> orderList,
			Map<String, AbstractTraceResult> results) {
		IDToNameMap APImap = NameToIDMap
				.getIDToNameMapfromNameToIDFile(Constants.RUNTIME_IDMAP_LOCATION);
		for (String key : orderList) {
			AbstractTraceResult traceObjectResult = results.get(key);
			System.out.println(traceObjectResult.toNameString(APImap));
		}

	}

	private static void showNumberOfSequences(List<String> orderList,
			Map<String, AbstractTraceResult> results) {
		for (String key : orderList) {
			AbstractTraceResult value = results.get(key);
			System.out.printf("%s Number of Sequences: %d \n", key, value
					.getNumberOfCalleeSequences());
		}
	}

	private static void writeToFile(ResultTable resultTable) {
		try {
			FileWriter fw = new FileWriter(Constants.OUTPUT_DIRECTORY
					+ "resultTable.txt");
			fw.write(resultTable.toString());
			fw.close();
		} catch (IOException e) {
			e.printStackTrace();
		}
	}

	/**
	 * Computes the Credibility for different results
	 *
	 * @param args
	 *            First argument indicates for which group the credibility
	 *            should be computed
	 * @throws FileNotFoundException
	 * @throws IOException
	 * @throws ClassNotFoundException
	 */
	public static void main(String[] args) throws FileNotFoundException,
			IOException, ClassNotFoundException {
		if (args.length > 0) {
			String firstArg = args[0].toLowerCase();
			if (firstArg.equals("png")) {
				System.out.println("Credibility for PNG readers");
				printCredibilityTable(ResultSets.getPngReaderResults(),
						ResultSets.getAllPngReaderResults());

			} else if (firstArg.equals("png2")) {
				System.out.println("Credibility for PNG with combined results");
				printCredibilityTable(ResultSets.getCombinedPngReaderResults(),
						ResultSets.getAllCombinedPngReaderResults());
			} else if (firstArg.equals("allpng")) {
				System.out.println("Credibility for all Png readers");
				printCredibilityTable(ResultSets.getPngResults(), ResultSets
						.getAllPngResults());
			} else if (firstArg.equals("xml")) {
				System.out.println("Credibility for xml results");
				Map<String, AbstractTraceResult> map = ResultSets
						.getCombinedSaxBenchResults();
				printCredibilityTable(map, ResultSets
						.getAllCombinedSaxBenchResults());
			} else if (firstArg.equals("saxbench")) {
				System.out.println("Credibility for SaxBench");
				printCredibilityTable(ResultSets.getSaxBenchResults(),
						ResultSets.getAllSaxBenchResults());
			} else if (firstArg.equals("saxbenchcombined")) {
				System.out.println("Credibility for SaxBench");
				printCredibilityTable(ResultSets.getCombinedSaxBenchResults(),
						ResultSets.getAllCombinedSaxBenchResults());
			} else if (firstArg.equals("saxtest")) {
				System.out.println("Credibility for Sax Test");
				printCredibilityTable(ResultSets.getSaxTestResults(),
						ResultSets.getAllSaxTestResults());
			}
			else if (firstArg.equals("xml2")) {
				System.out.println("Credibility for Sax Test and SaxBench");
				Map<String, AbstractTraceResult> firstMap = ResultSets
						.getSaxTestResults();
				firstMap.putAll(ResultSets.getCombinedSaxBenchResults());
				Map<String, AbstractTraceResult> compareMap = ResultSets
						.getAllSaxTestResults();
				compareMap.putAll(ResultSets.getAllCombinedSaxBenchResults());
				printCredibilityTable(firstMap, compareMap);
			} else {
				System.out.println("No Option" + firstArg);
			}
		}
	}

}
