package org.softevo.apibirthmark.tracing.tracehandler;

import java.io.IOException;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.Map;
import java.util.Set;
import java.util.SortedSet;
import java.util.TreeSet;

import org.softevo.apibirthmark.tracing.Tracer;
import org.softevo.apibirthmark.tracing.Tracer.TraceObject;
import org.softevo.apibirthmark.tracing.mapping.IDMapError;
import org.softevo.apibirthmark.tracing.mapping.IDToNameMap;
import org.softevo.apibirthmark.tracing.mapping.NameToIDMap;




/**
 * The CallCounter class counts outgoing calls for every class
 * @author schuler
 *
 */
public class CallCounter implements TraceObjectHandler {

	private Map<Integer,ClassCounter> classHashtable = new Hashtable<Integer,ClassCounter>();

	public void handle(TraceObject traceObject) {
		ClassCounter classCounter;
		Integer calleeClassInteger = new Integer(traceObject.calleeClassID);
		if (!classHashtable.containsKey(calleeClassInteger)) {
			classCounter = new ClassCounter(1);
			classHashtable.put(calleeClassInteger, classCounter);
		} else {
			classCounter = (ClassCounter) classHashtable
					.get(calleeClassInteger);
		}
		classCounter.counter++;
	}

	public void report() {
		Set set = classHashtable.entrySet();
		SortedSet<ClassSetEntry> sortedSet = new TreeSet<ClassSetEntry>();
		for (Iterator iter = set.iterator(); iter.hasNext();) {
			Map.Entry element = (Map.Entry) iter.next();
			Integer classID = (Integer) element.getKey();
			ClassCounter classCounter = (ClassCounter) element.getValue();
			sortedSet.add(new ClassSetEntry(classID.intValue(),
					classCounter.counter));
		}
		IDToNameMap idToNameMap = null;
		String location = System.getProperty("idFile");
		if (location != null) {
			try {
				NameToIDMap idMap = NameToIDMap
						.getFromFile(location);
				idToNameMap = idMap.getIDtoNameMap();
			} catch (IOException e) {
				e.printStackTrace();
			}
		}

		for (Iterator iter = sortedSet.iterator(); iter.hasNext();) {
			ClassSetEntry element = (ClassSetEntry) iter.next();
			String className = "class not found";
			if (idToNameMap != null) {
				try {
					className = idToNameMap.getClassnameByID(element.classID);
				} catch (IDMapError e) {
					e.printStackTrace();
				}
			}
			Tracer.mySysOut.println("ClassID: " + element.classID + "("
					+ className + ")\n\t " + element.objects +" calls");
		}
	}

	private static class ClassCounter {
		public int counter;

		public ClassCounter(int i) {
			counter = i;
		}

	}

	private static class ClassSetEntry implements Comparable {

		int classID;

		int objects;

		public ClassSetEntry(int classID, int counter) {
			this.classID = classID;
			objects = counter;
		}

		public int compareTo(Object o) {
			return objects - ((ClassSetEntry) o).objects;
		}

	}

}
