package org.softevo.apibirthmark.tracing.tracehandler;

import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.HashSet;
import java.util.Set;

import org.softevo.apibirthmark.constants.Constants;

import org.softevo.apibirthmark.tracing.Tracer;
import org.softevo.apibirthmark.tracing.Tracer.TraceObject;
import org.softevo.apibirthmark.util.ExcludeList;
import org.softevo.apibirthmark.util.ObjectIO;

/**
 * ExcludeHandler acts as an adapter. It forwards calls to a given TraceObjectHandler if he call is not
 * in a list of calls to be excluded.
 *
 * @author schuler
 *
 */
public class ExcludeHandler implements TraceObjectHandler {

	/**
	 * TraceObjectHandler to forward the calls
	 */
	private TraceObjectHandler traceObjectHandler;

	/**
	 * Set of classIDs. Calls to these classes are filtered
	 */
	private Set<Integer> excludeList;

	/**
	 * Number of Excluded Calls
	 */
	private int excludedCalls;

	public ExcludeHandler(TraceObjectHandler traceObjectHandler) {
		this.traceObjectHandler = traceObjectHandler;
		ExcludeList excludeIDList;
		try {
			excludeIDList = (ExcludeList) ObjectIO
					.getFromFile(Constants.EXCLUDE_FILE);
			excludeList = new HashSet<Integer>();
			Tracer.mySysOut.println("Exclude IDs");
			for (int i : excludeIDList.getExcludedClassIDs()) {
				excludeList.add(i);
				Tracer.mySysOut.println(i);
			}
		} catch (FileNotFoundException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		} catch (ClassNotFoundException e) {
			e.printStackTrace();
		}
	}

	/**
	 * If call should not be excluded it is forwarded to the corresponding
	 * TraceObjectHandler
	 *
	 * @see tracing.tracehandler.TraceObjectHandler#handle(tracing.Tracer.TraceObject)
	 */
	public void handle(TraceObject traceObject) {
		if (excludeList.contains(traceObject.calleeClassID)) {
			excludedCalls++;
			return;
		}
		traceObjectHandler.handle(traceObject);
	}

	/* (non-Javadoc)
	 * @see tracing.tracehandler.TraceObjectHandler#report()
	 */
	public void report() {
		Tracer.mySysOut.println("Excluded Calls: " + excludedCalls);
		traceObjectHandler.report();
	}

}
