package org.softevo.apibirthmark.tracing.tracehandler;

import java.util.HashMap;
import java.util.Map.Entry;

import org.softevo.apibirthmark.constants.Constants;

import org.softevo.apibirthmark.tracing.Tracer;
import org.softevo.apibirthmark.tracing.Tracer.TraceObject;
import org.softevo.apibirthmark.tracing.result.TraceObjectResult;
import org.softevo.apibirthmark.tracing.tracehandler.window.TraceObjectWindow;
import org.softevo.apibirthmark.tracing.tracehandler.window.container.HashWindowContainer;
import org.softevo.apibirthmark.util.ObjectIO;

public class WindowCallHandler implements TraceObjectHandler {

	protected HashMap<Integer, TraceObjectWindow> sequenceMap = new HashMap<Integer, TraceObjectWindow>();

	protected HashMap<Integer, TraceObjectWindow> staticHashMap = new HashMap<Integer, TraceObjectWindow>();

	protected HashWindowContainer windowContainer = new HashWindowContainer();

	protected boolean ignoreStaticSequences = true;

	public void handle(TraceObject traceObject) {
		Tracer.mySysOut.println("Call ignored");
	}

	public void handle(TraceObject traceObject, int key){
		TraceObjectWindow window;
		synchronized (sequenceMap) {
			HashMap<Integer, TraceObjectWindow> windowHashMap = sequenceMap;
			if (key == Constants.STATICOBJECTID) {
				if (ignoreStaticSequences) {
					return;
				}
				key = traceObject.callerClassID;
				windowHashMap = staticHashMap;
			}
			if (windowHashMap.containsKey(key)) {
				window = windowHashMap.get(key);
			} else {
				window = new TraceObjectWindow();
				windowHashMap.put(key, window);
			}
			window.addCall(traceObject);
			if (window.isFull()) {
				windowContainer.add(window);
			}
		}
	}
	
	public synchronized void report() {
		fillSequences();
		TraceObjectResult abstractTraceResult = windowContainer
				.getTraceResult();
		Tracer.mySysOut.println("Sequences: "+ abstractTraceResult.getNumberOfSequences());
		Tracer.mySysOut.println("Callee Sequences: "+ abstractTraceResult.getNumberOfCalleeSequences());
		Tracer.mySysOut.println("Writing Result");
		ObjectIO.saveToFile(Tracer.resultFileLocation, abstractTraceResult);
		windowContainer.setCanShutdown(true);
	}

	protected void fillSequences() {
		synchronized (sequenceMap) {
			for (Entry<Integer, TraceObjectWindow> element : sequenceMap
					.entrySet()) {
				TraceObjectWindow window = element.getValue();
				if (!window.isFull()) {
					window.fill();
					windowContainer.add(window);
				}
			}
		}
	}
}
