package org.softevo.apibirthmark.tracing.tracehandler.window;

import java.io.Serializable;

import org.softevo.apibirthmark.tracing.mapping.IDToNameMap;

/**
 * AbstractResultWindow is an abstract class for holding a traced call sequence. The caller and callee
 * classes and methods and objects are represented with IDs.
 *
 * @author schuler
 *
 */
public interface AbstractResultWindow extends Comparable, Serializable {

	/**
	 * Return a String where the IDs of describing the method calls are replaced
	 * with their actual names if the ID can be found in the idToNameMap
	 *
	 * @param idToNameMap
	 * @return
	 */
	public String toNameString(IDToNameMap idToNameMap);

	/**
	 * Return the sequence of callee methods.
	 *
	 * @return
	 */
	public int[] getCalleeSequence();

	/**
	 * Return the sequence of caller methods.
	 *
	 * @return
	 */
	public int[] getCallerMethodSequence();

	/**
	 * Return the sequence of caller classes.
	 *
	 * @return
	 */
	public int[] getCallerClassSequence();

	/**
	 * Assign a weight to the sequence.
	 *
	 * @param weight
	 */
	public void setWeight(int weight);

	/**
	 * Return he assigned weight of the sequence.
	 *
	 * @return
	 */
	public int getWeight();


	/**
	 * @return 
	 */
	public AbstractResultWindow getCaleeWindow();

	/**
	 * Add a weight to this sequence.
	 *
	 * @param add
	 */
	public void addWeight(int add);

	/**
	 * Checks if given classID is contained in sequence.
	 *
	 * @param classID
	 * @return
	 */
	public boolean containsClassID(int classID);

	/**
	 * Return size of sequence
	 *
	 * @return
	 */
	public int getSize();

	/**
	 * Return the number of different callee methods contained in this sequence
	 *
	 * @return
	 */
	public int getNumberOfDifferentCalleeMethods();

	/**
	 * Return the number of different callee classes in this sequence
	 *
	 * @return
	 */
	public int getNumberOfDifferentCalleeClasses();

}
