package org.softevo.apibirthmark.tracing.tracehandler.window;

import java.util.Vector;

import org.softevo.apibirthmark.constants.Constants;

import org.softevo.apibirthmark.tracing.Tracer.TraceObject;
import org.softevo.apibirthmark.tracing.mapping.IDMapError;
import org.softevo.apibirthmark.tracing.mapping.IDToNameMap;

public class TraceObjectResultWindow implements AbstractResultWindow {

	private static final long serialVersionUID = 2L;

	private int weight = 1;

	private TraceObject[] traceObjectSequence;

	public TraceObjectResultWindow(TraceObject[] traceObjects) {
		this.traceObjectSequence = traceObjects;
	}

	/*
	 * (non-Javadoc)
	 *
	 * @see tracing.tracehandler.window.AbstractResultWindow#toNameString(tracing.mapping.IDToNameMap)
	 */
	public String toNameString(IDToNameMap idToNameMap) {
		StringBuilder buffer = new StringBuilder();
		buffer.append("caller: ");
		for (TraceObject traceObject : traceObjectSequence) {
			try {
				buffer.append(idToNameMap
						.getClassnameByID(traceObject.callerClassID));
				buffer.append(' ');
				buffer.append(idToNameMap.getMethodnameByID(
						traceObject.callerClassID, traceObject.callerMethodID));
			} catch (IDMapError e) {
				buffer.append(traceObject.callerClassID + " "
						+ traceObject.callerMethodID);
			}
			buffer.append(" - ");
		}
		buffer.append(" \ncallee: ");
		for (TraceObject traceObject : traceObjectSequence) {
			try {
				buffer.append(idToNameMap
						.getClassnameByID(traceObject.calleeClassID));
				buffer.append(' ');
				buffer.append(idToNameMap.getMethodnameByID(
						traceObject.calleeClassID, traceObject.calleeMethodID));
			} catch (IDMapError e) {
				buffer.append(traceObject.calleeClassID + " "
						+ traceObject.calleeMethodID);
			}
			buffer.append(" - ");
		}
		return buffer.toString();
	}

	/*
	 * (non-Javadoc)
	 *
	 * @see java.lang.Comparable#compareTo(java.lang.Object)
	 */
	public int compareTo(Object o) {
		TraceObjectResultWindow compareWindow = (TraceObjectResultWindow) o;
		if (compareWindow.traceObjectSequence.length == traceObjectSequence.length) {
			for (int i = 0; i < traceObjectSequence.length; i++) {
				if (traceObjectSequence[i].calleeMethodID != compareWindow.traceObjectSequence[i].calleeMethodID) {
					return traceObjectSequence[i].calleeMethodID
							- compareWindow.traceObjectSequence[i].calleeMethodID;
				}
				if (traceObjectSequence[i].callerMethodID != compareWindow.traceObjectSequence[i].callerMethodID) {
					return traceObjectSequence[i].callerMethodID
							- compareWindow.traceObjectSequence[i].callerMethodID;
				}
			}
			return 0;
		} else {
			throw new RuntimeException("Different Length in Results");
			// return traceObjectSequence.length
			// - compareWindow.traceObjectSequence.length;
		}
	}

	/*
	 * (non-Javadoc)
	 *
	 * @see java.lang.Object#equals(java.lang.Object)
	 */
	@Override
	public boolean equals(Object obj) {
		if (traceObjectSequence == null || obj == null) {
			return false;
		}
		TraceObjectResultWindow compareWindow = (TraceObjectResultWindow) obj;
		if (compareWindow.traceObjectSequence.length == traceObjectSequence.length) {
			for (int i = 0; i < traceObjectSequence.length; i++) {
				if (traceObjectSequence[i].calleeMethodID != compareWindow.traceObjectSequence[i].calleeMethodID) {
					return false;
				}
				if (traceObjectSequence[i].callerMethodID != compareWindow.traceObjectSequence[i].callerMethodID) {
					return false;
				}
			}
			return true;
		} else {
			return false;
		}
	}

	/*
	 * (non-Javadoc)
	 *
	 * @see java.lang.Object#hashCode()
	 */
	@Override
	public int hashCode() {
		if (traceObjectSequence != null) {
			return traceObjectSequence[0].calleeMethodID + 37
					* traceObjectSequence[0].callerMethodID;
		}
		return super.hashCode();
	}

	/*
	 * (non-Javadoc)
	 *
	 * @see tracing.tracehandler.window.AbstractResultWindow#getCalleeSequence()
	 */
	public int[] getCalleeSequence() {
		int[] calleeSequence = new int[traceObjectSequence.length];
		for (int i = 0; i < traceObjectSequence.length; i++) {
			calleeSequence[i] = traceObjectSequence[i].calleeMethodID;
		}
		return calleeSequence;
	}

	/*
	 * (non-Javadoc)
	 *
	 * @see tracing.tracehandler.window.AbstractResultWindow#getCallerSequence()
	 */
	public int[] getCallerMethodSequence() {
		int[] callerSequence = new int[traceObjectSequence.length];
		for (int i = 0; i < traceObjectSequence.length; i++) {
			callerSequence[i] = traceObjectSequence[i].callerMethodID;
		}
		return callerSequence;
	}

	/*
	 * (non-Javadoc)
	 *
	 * @see java.lang.Object#toString()
	 */
	@Override
	public String toString() {
		StringBuilder buffer = new StringBuilder();
		for (TraceObject traceObject : traceObjectSequence) {
			buffer.append(traceObject.calleeClassID);
			buffer.append('-');
		}
		buffer.append("  ");
		for (TraceObject traceObject : traceObjectSequence) {
			buffer.append(traceObject.callerClassID);
			buffer.append('-');
		}
		return buffer.toString();
	}

	/*
	 * (non-Javadoc)
	 *
	 * @see tracing.tracehandler.window.AbstractResultWindow#getCallerClassSequence()
	 */
	public int[] getCallerClassSequence() {
		int[] callerSequence = new int[traceObjectSequence.length];
		for (int i = 0; i < traceObjectSequence.length; i++) {
			callerSequence[i] = traceObjectSequence[i].callerClassID;
		}
		return callerSequence;
	}

	/*
	 * (non-Javadoc)
	 *
	 * @see tracing.tracehandler.window.AbstractResultWindow#getWeight()
	 */
	public int getWeight() {
		return weight;
	}

	/*
	 * (non-Javadoc)
	 *
	 * @see tracing.tracehandler.window.AbstractResultWindow#setWeight(int)
	 */
	public void setWeight(int weight) {
		this.weight = weight;
	}

	/*
	 * (non-Javadoc)
	 *
	 * @see tracing.tracehandler.window.AbstractResultWindow#addWeight(int)
	 */
	public void addWeight(int add) {
		weight += add;
	}

	/*
	 * (non-Javadoc)
	 *
	 * @see tracing.tracehandler.window.AbstractResultWindow#getCaleeWindow()
	 */
	public AbstractResultWindow getCaleeWindow() {
		TraceObject[] sequence = new TraceObject[traceObjectSequence.length];
		for (int i = 0; i < traceObjectSequence.length; i++) {
			sequence[i] = new TraceObject(0, 0, 0,
					traceObjectSequence[i].calleeClassID,
					traceObjectSequence[i].calleeMethodID, 0);
		}
		return new TraceObjectResultWindow(sequence);
	}

	/*
	 * (non-Javadoc)
	 *
	 * @see tracing.tracehandler.window.AbstractResultWindow#containsClassID(int)
	 */
	public boolean containsClassID(int classID) {
		for (TraceObject traceObject : traceObjectSequence) {
			if (traceObject.calleeClassID == classID
					|| traceObject.callerClassID == classID) {
				return true;
			}
		}
		return false;
	}

	/*
	 * (non-Javadoc)
	 *
	 * @see tracing.tracehandler.window.AbstractResultWindow#getSize()
	 */
	public int getSize() {
		return traceObjectSequence.length;
	}

	/*
	 * (non-Javadoc)
	 *
	 * @see tracing.tracehandler.window.AbstractResultWindow#getNumberOfDifferentCalleeMethods()
	 */
	public int getNumberOfDifferentCalleeMethods() {
		Vector<Integer> vector = new Vector<Integer>();
		int numberOfDifferentCalleeMethods = 0;
		for (TraceObject traceObject : traceObjectSequence) {
			int methodID = traceObject.calleeMethodID;
			if (!vector.contains(methodID)) {
				vector.add(methodID);
				numberOfDifferentCalleeMethods++;
			}
		}
		return numberOfDifferentCalleeMethods;
	}

	/* (non-Javadoc)
	 * @see tracing.tracehandler.window.AbstractResultWindow#getNumberOfDifferentCalleeClasses()
	 */
	public int getNumberOfDifferentCalleeClasses() {
		Vector<Integer> vector = new Vector<Integer>();
		int numberOfDifferentCalleeClasses = 0;
		for (TraceObject traceObject : traceObjectSequence) {
			int classID = traceObject.calleeClassID;
			if (!vector.contains(classID) && classID != Constants.FIll_CLASS_ID) {
				vector.add(classID);
				numberOfDifferentCalleeClasses++;
			}
		}
		return numberOfDifferentCalleeClasses;
	}


}
