package org.softevo.apibirthmark.tracing.tracehandler.window.container;

import java.util.HashSet;

import org.softevo.apibirthmark.tracing.Tracer;
import org.softevo.apibirthmark.tracing.result.TraceObjectResult;
import org.softevo.apibirthmark.tracing.tracehandler.window.AbstractResultWindow;
import org.softevo.apibirthmark.tracing.tracehandler.window.AbstractWindow;

/**
 * Implementation of AbstractWindowthat uses a HashSet to store the sequences
 *
 * @author schuler
 *
 */
public class HashWindowContainer implements AbstractWindowContainer {

	private HashSet<AbstractResultWindow> resultSet = new HashSet<AbstractResultWindow>();

	private boolean lock = false;

	private boolean canShutdown = false;

	private int missingCalls = 0;

	private int sameWindows = 0;

	/*
	 * (non-Javadoc)
	 *
	 * @see tracing.tracehandler.window.container.AbstractWindowContainer#add(tracing.tracehandler.window.AbstractWindow)
	 */
	public synchronized void add(AbstractWindow w) {
		if (lock) {
			missingCalls++;
			if (missingCalls % 10000 == 0) {
				Tracer.mySysOut.println("Missing calls:" + missingCalls);
				if (canShutdown) {
					Tracer.mySysOut.println("Forcing shutdown");
					// can only get here if JVM is trying to shut down and
					// blocked by a thread, so force termination
					Runtime.getRuntime().halt(0);
				}
			}
			return;
		}
		AbstractResultWindow resultWindow = w.getResultWindow();
		if (!resultSet.contains(resultWindow)) {
			resultSet.add(resultWindow);
		} else {
			sameWindows++;
		}
	}

	/*
	 * (non-Javadoc)
	 *
	 * @see tracing.tracehandler.window.container.AbstractWindowContainer#getTraceResult()
	 */
	public synchronized TraceObjectResult getTraceResult() {
		lock = true;
		Tracer.mySysErr.println("Discarded windows " + sameWindows);
		AbstractResultWindow[] resultWindow = resultSet
				.toArray(new AbstractResultWindow[0]);
		Tracer.mySysErr.println("Generating Result");
		TraceObjectResult traceObjectResult = new TraceObjectResult(
				resultWindow);
		return traceObjectResult;
	}

	/**
	 * Method to force the shutdown, must be used in rare cases when JVM does
	 * not shutdown properly (when shutdown is blocked by a thread)
	 *
	 * @param canShutdown
	 */
	public void setCanShutdown(boolean canShutdown) {
		this.canShutdown = canShutdown;
	}
}
