package org.softevo.apibirthmark.transformation.bytecodeTransformer;

import org.objectweb.asm.ClassReader;
import org.objectweb.asm.ClassVisitor;
import org.objectweb.asm.ClassWriter;

/**
 * Class does instrumentation of bytecode by using appropriate ASM ClassVisitor
 *
 * @author schuler
 *
 */
public abstract class BytecodeTransformer {

	protected ClassWriter cw;

	/**
	 * Provide a ASM chain to do the instrumentation
	 *
	 * @return
	 */
	protected abstract ClassVisitor getTransformer();

	/**
	 * Transforms the given bytecode
	 *
	 * @param bytecode
	 *            original bytecode
	 * @return transformed bytecode
	 */
	public byte[] transformBytecode(byte[] bytecode) {
		ClassReader cr = new ClassReader(bytecode);
		ClassVisitor cv = getTransformer();
		cr.accept(cv, false);
		return cw.toByteArray();
	}
}
