package org.softevo.apibirthmark.transformation.bytecodeTransformer;

import org.objectweb.asm.ClassVisitor;
import org.objectweb.asm.ClassWriter;
import org.objectweb.asm.util.CheckClassAdapter;

import org.softevo.apibirthmark.tracing.mapping.NameToIDMap;
import org.softevo.apibirthmark.asmadapters.proxies.classHierarchy.ClassHierarchy;
import org.softevo.apibirthmark.asmadapters.proxies.observed.ObservedProxyClassAdapter;

/**
 * Provides the ASM chain to instrument calls to the API (replace them with
 * calls to proxy methods)
 *
 * @author schuler
 *
 */
public class ObservedTransformer extends BytecodeTransformer {

	private NameToIDMap relevantMap;

	private ClassHierarchy classHierarchy;

	private NameToIDMap idMap;

	/**
	 * @param idMap
	 *            IDMap that contains the IDs of the to instrumented classes and
	 *            methods
	 * @param relevantMap
	 *            IDMap that contains the methods that should be instrumented
	 * @param classHierarchy
	 *            Class Hierarchy to get super classes if API classes are
	 *            sub classed
	 */
	public ObservedTransformer(NameToIDMap idMap, NameToIDMap relevantMap,
			ClassHierarchy classHierarchy) {
		super();
		this.idMap = idMap;
		this.relevantMap = relevantMap;
		this.classHierarchy = classHierarchy;
	}

	/*
	 * (non-Javadoc)
	 *
	 * @see transformation.bytecodeTransformer.BytecodeTransformer#getTransformer()
	 */
	protected ClassVisitor getTransformer() {
		cw = new ClassWriter(true);
		ClassVisitor cc = new CheckClassAdapter(cw);
		ObservedProxyClassAdapter observedProxyClassAdapter = new ObservedProxyClassAdapter(
				cc, idMap, relevantMap, classHierarchy);
		return observedProxyClassAdapter;
	}

}
