package org.softevo.apibirthmark.transformation.bytecodeTransformer;

import org.objectweb.asm.ClassVisitor;
import org.objectweb.asm.ClassWriter;
import org.objectweb.asm.util.CheckClassAdapter;

import org.softevo.apibirthmark.tracing.mapping.NameToIDMap;
import org.softevo.apibirthmark.asmadapters.proxies.GenerateProxyClassAdapter;

/**
 * ProxyTransformer is a BytecodeTransformer that inserts proxy method in the
 * transformed classes
 *
 * @author schuler
 *
 */
public class ProxyTransformer extends BytecodeTransformer {

	private NameToIDMap idMap;

	/**
	 * Generates new ProxyTransformer IDs that are computed during
	 * instrumentation are inserted into given NameToIDMap
	 *
	 * @param idMap
	 *            NameToIDMap where the IDs of the instrumented methods and
	 *            classes are inserted
	 */
	public ProxyTransformer(NameToIDMap idMap) {
		super();
		this.idMap = idMap;
	}

	/* (non-Javadoc)
	 * @see transformation.bytecodeTransformer.BytecodeTransformer#getTransformer()
	 */
	protected ClassVisitor getTransformer() {
		cw = new ClassWriter(true);
		ClassVisitor cc = new CheckClassAdapter(cw);
		GenerateProxyClassAdapter generateProxyClassAdapter = new GenerateProxyClassAdapter(
				cc, idMap);
		return generateProxyClassAdapter;
	}
}
