package org.softevo.apibirthmark.util;

import java.io.IOException;
import java.io.Serializable;
import java.util.Arrays;

import org.apache.log4j.Logger;

import org.softevo.apibirthmark.tracing.mapping.NameToIDMap;
import org.softevo.apibirthmark.constants.Constants;

/**
 * Provides a list of class IDs that should be excluded from tracing.
 *
 * @author schuler
 *
 */
public class ExcludeList implements Serializable {

	private static Logger logger = Logger.getLogger(ExcludeList.class);

	private static final long serialVersionUID = 1L;

	private int[] excludedClassIDs;

	/**
	 * Creates a file that contains the IDs of the excluded classes, such that
	 * this file can be read when tracing a program run rather than computing
	 * the IDs for each run again.
	 */
	public static void createExcludeFile() {
		logger.info("Reading runtime idmap from "
				+ Constants.RUNTIME_IDMAP_LOCATION);
		try {
			NameToIDMap nameToIDMap = NameToIDMap
					.getFromFile(Constants.RUNTIME_IDMAP_LOCATION);
			int[] excludeIDs = new int[Constants.excludedClasses.length];
			for (int i = 0; i < excludeIDs.length; i++) {
				excludeIDs[i] = (nameToIDMap
						.getClassID(Constants.excludedClasses[i]));

			}
			logger.info("Excluded IDs" + Arrays.toString(excludeIDs));
			ExcludeList excludeList = new ExcludeList();
			excludeList.setExcludedClassIDs(excludeIDs);
			ObjectIO.saveToFile(Constants.EXCLUDE_FILE, excludeList);
		} catch (IOException e) {
			e.printStackTrace();
		}
	}

	/**
	 * Return the IDs of the classes to exclude when tracing.
	 *
	 * @return An array that contains the IDs of the classes to exclude.
	 */
	public int[] getExcludedClassIDs() {
		return excludedClassIDs;
	}

	/**
	 * Set the IDs of the excluded classes/
	 *
	 * @param excludedClassIDs
	 *            Array that contains the classes to exclude from tracing.
	 */
	public void setExcludedClassIDs(int[] excludedClassIDs) {
		this.excludedClassIDs = excludedClassIDs;
	}

	public static void main(String[] args) {
		createExcludeFile();
	}
}
