package org.softevo.apibirthmark.util;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;

import org.apache.log4j.Logger;

import org.softevo.apibirthmark.tracing.result.TraceObjectResult;

/**
 * Class used to write and read objects to/from files.
 * 
 * @author schuler
 * 
 */
public class ObjectIO {

	private static Logger logger = Logger.getLogger(ObjectIO.class);

	/**
	 * Save an object to a file.
	 * 
	 * @param filename
	 *            Name of the file the object should be saved to.
	 * @param object
	 *            The object to save.
	 */
	public static void saveToFile(String filename, Object object) {
		File file = new File(filename);
		saveToFile(file, object);
	}

	/**
	 * Save an object to a file.
	 * 
	 * @param file
	 *            File the object should be saved to.
	 * @param object
	 *            The object to save.
	 */
	public static void saveToFile(File file, Object object) {
		try {
			logger.info("Writing: " + file);
			ObjectOutputStream objectOut = new ObjectOutputStream(
					new FileOutputStream(file));
			objectOut.writeObject(object);
			objectOut.close();
		} catch (IOException e) {
			e.printStackTrace();
			throw new RuntimeException("Couldn't save Object" + object
					+ " to file " + file);
		}
	}

	/**
	 * Reads an object from the given file
	 * 
	 * @param filename
	 *            Name of the file
	 * @return The Object that was read from file
	 * @throws FileNotFoundException
	 * @throws IOException
	 * @throws ClassNotFoundException
	 */
	public static Object getFromFile(String filename)
			throws FileNotFoundException, IOException, ClassNotFoundException {
		File file = new File(filename);
		return getFromFile(file);
	}

	/**
	 * Reads an object from the given file
	 * 
	 * @param file
	 *            File to read the Object from
	 * @return The Object that was read from file
	 * @throws ClassNotFoundException
	 * @throws FileNotFoundException
	 * @throws IOException
	 */
	public static Object getFromFile(File file) throws ClassNotFoundException,
			FileNotFoundException, IOException {
		ObjectInputStream objectInputStream = new ObjectInputStream(
				new FileInputStream(file));
		Object returnObject = objectInputStream.readObject();
		objectInputStream.close();
		return returnObject;
	}

	/**
	 * Return the TraceObjectResult from given file.
	 * 
	 * @param filename
	 *            Name of the file that contains the TraceObjectResult
	 * @return The TraceObjectResult from given file.
	 */
	public static TraceObjectResult getTraceObjectResult(String filename) {
		try {
			return (TraceObjectResult) getFromFile(filename);
		} catch (FileNotFoundException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		} catch (ClassNotFoundException e) {
			e.printStackTrace();
		}
		return null;
	}

}
