package org.softevo.apibirthmark.util;

import java.util.Iterator;
import java.util.Vector;

import org.apache.commons.configuration.ConfigurationException;
import org.apache.commons.configuration.PropertiesConfiguration;
import org.apache.log4j.Logger;


/**
 * Reads data from a property file. Thus, data can be shared between ant files
 * and the program.
 *
 * @author schuler
 *
 */
public class Property {

	private static Logger logger = Logger.getLogger(Property.class);

	private static String USER_HOME_KEY = "user.home";

	private PropertiesConfiguration propertiesConfiguration;

	private static Property property;

	static {
		property = new Property();
	}

	/**
	 * Create a new Property object for projects property file
	 */
	private Property() {
		try {
			propertiesConfiguration = new PropertiesConfiguration(PropertyFileLoader.getPropertyFileLocation());
		} catch (ConfigurationException e) {
			e.printStackTrace();
		}
		propertiesConfiguration.setProperty(USER_HOME_KEY, System.getProperty(USER_HOME_KEY));
	}

	/**
	 * Return the property of given key
	 *
	 * @param key
	 *            The key of the property
	 * @return The value of the property
	 */
	private String getProjectProperty(String key) {
		return propertiesConfiguration.getString(key);
	}

	/**
	 * static method that returns the property
	 *
	 * @param key
	 *            The key of the property
	 * @return The value of the property
	 */
	public static String getProperty(String key) {
		String value = property.getProjectProperty(key);
		logger.debug("Loading Property: " + key + " with value: " + value);
		return value;
	}

	/**
	 * Return all keys of projects property file
	 *
	 * @return all keys of projects property file
	 */
	public static String[] getKeys() {
		return property.getPropKeys();
	}

	private String[] getPropKeys() {
		Iterator iterator = propertiesConfiguration.getKeys();
		Vector<String> keys = new Vector<String>();
		while (iterator.hasNext()) {
			keys.add((String) iterator.next());
		}
		return keys.toArray(new String[0]);
	}

	/**
	 * Return all keys that contain given part
	 *
	 * @param part
	 *            substring that must be contained in the key
	 * @return all keys that contain the part
	 */
	public static String[] getKeys(String part) {
		String[] keys = property.getPropKeys();
		Vector<String> resultKeys = new Vector<String>();
		for (String string : keys) {
			if (string.contains(part)) {
				resultKeys.add(string);
			}
		}
		return resultKeys.toArray(new String[0]);
	}
}
