package org.softevo.apibirthmark.util;

import java.io.FileNotFoundException;
import java.io.IOException;

import org.softevo.apibirthmark.tracing.mapping.IDToNameMap;
import org.softevo.apibirthmark.tracing.mapping.NameToIDMap;
import org.softevo.apibirthmark.tracing.result.TraceObjectResult;

/**
 * Takes a trace result (location of file) and a NameToIDMap as arguments and
 * prints the trace result with IDs converted to names where possible.
 *
 * @author schuler
 *
 */
public class ShowTrace {

	/**
	 * Prints the call sequence set of a result read from a file. If possible,
	 * the IDs are resolved to names in given NameToIDMap (also read from a
	 * file)
	 *
	 * @param args
	 *            first argument is the file that contains the TraceObjectResult
	 *            and the second the one that contains the NameToIDMap to
	 *            resolve the IDs to method names
	 * @throws FileNotFoundException
	 * @throws IOException
	 * @throws ClassNotFoundException
	 */
	public static void main(String[] args) throws FileNotFoundException,
			IOException, ClassNotFoundException {
		if (args.length >= 2) {
			showTrace(args[0], args[1]);
		} else {
			System.out.println("Specify result file and file for NameToIDMap");
		}
	}

	/**
	 * Print all sequences contained in TraceObjectResult were the names are
	 * used if they can be resolved with given IDToNameMap.
	 *
	 * @param traceObjectResult
	 *            Location of the TraceObjectResult file that contains the
	 *            sequences.
	 * @param idMap
	 *            Location of the IDToNameMap file to resolve the method names.
	 */
	private static void showTrace(String resultFilename,
			String nameToIDMapFilename) throws FileNotFoundException,
			IOException, ClassNotFoundException {
		TraceObjectResult traceObjectResult = (TraceObjectResult) ObjectIO
				.getFromFile(resultFilename);
		IDToNameMap map = NameToIDMap
				.getIDToNameMapfromNameToIDFile(nameToIDMapFilename);
		showTrace(traceObjectResult, map);
	}

	/**
	 * Print all sequences contained in TraceObjectResult were the names are
	 * used if they can be resolved with given IDToNameMap.
	 *
	 * @param traceObjectResult
	 *            The TraceObjectResult that contains the sequences.
	 * @param idMap
	 *            IDToNameMap to resolve the method names.
	 */
	public static void showTrace(TraceObjectResult traceObjectResult,
			IDToNameMap idMap) {
		System.out.println(traceObjectResult.toNameString(idMap));
	}
}
