package org.softevo.apibirthmark.validation;

import java.util.HashMap;
import java.util.Map;
import java.util.Vector;

import org.softevo.apibirthmark.tracing.Tracer;
import org.softevo.apibirthmark.tracing.Tracer.TraceObject;
import org.softevo.apibirthmark.util.ObjectIO;
import org.softevo.apibirthmark.validation.CallGraph.APIMethodNode;
import org.softevo.apibirthmark.validation.CallGraph.CallingMethodNode;
import org.softevo.apibirthmark.validation.CallGraph.CallingObjectNode;
import org.softevo.apibirthmark.validation.CallGraph.MainCallingClass;
import org.softevo.apibirthmark.validation.CallGraph.MainObjectNode;

/**
 * Generates the expected trace from the API call graph
 * @author schuler
 *
 */
public class TraceGenerator {

	private Map<String, Integer> callingObjectHashTable = new HashMap<String, Integer>();

	private Map<String, Integer> apiObjectHashTable = new HashMap<String, Integer>();

	private Vector<TraceObject> traceObjectVector = new Vector<TraceObject>();

	private int calleeClassID;

	private int calleeMethodID;

	private int calleeObjectID;

	private int callerClassID;

	private int callerMethodID;

	private int callerObjectID;

	/**
	 * generate a trace for given API Call Graph (represented by
	 * mainCallingClass)
	 *
	 * @param mainCallingClass
	 *            Name of the main class
	 */
	public void generateExpectedTrace(MainCallingClass mainCallingClass) {
		MainObjectNode[] objects = mainCallingClass.objects;
		for (int i = 0; i < objects.length; i++) {
			callerObjectID = addCallingObject(objects[i].type);
			callerClassID = objects[i].callingClassNode.id;
			generateTrace(objects[i]);
		}
	}

	private int addCallingObject(String objectType) {
		if (callingObjectHashTable.containsKey(objectType)) {
			int i = callingObjectHashTable.get(objectType);
			i++;
			callingObjectHashTable.put(objectType, i);
			return i;
		} else {
			callingObjectHashTable.put(objectType, 0);
		}
		return 0;
	}

	private void generateTrace(MainObjectNode node) {
		CallingMethodNode[] methods = node.callingMethodNodes;
		for (int i = 0; i < methods.length; i++) {
			generateTrace(methods[i]);
		}
	}

	private void generateTrace(CallingMethodNode node) {
		CallingObjectNode[] objects = node.callingObjects;
		callerMethodID = node.id;
		for (int i = 0; i < objects.length; i++) {
			calleeClassID = objects[i].apiClassNode.id;
			generateTrace(objects[i]);
		}
	}

	int count = 0;

	private void generateTrace(CallingObjectNode node) {
		APIMethodNode[] apiMethods = node.methods;
		calleeObjectID = addAPIObject(node.type);
		traceObjectVector.add(new Tracer.TraceObject(callerClassID,
				callerMethodID, callerObjectID, calleeClassID,
				node.apiClassNode.constructorID, calleeObjectID));
		for (int i = 0; i < apiMethods.length; i++) {
			calleeMethodID = apiMethods[i].id;
			traceObjectVector.add(new Tracer.TraceObject(callerClassID,
					callerMethodID, callerObjectID, calleeClassID,
					calleeMethodID, calleeObjectID));
		}
	}

	private int addAPIObject(String objectType) {
		if (apiObjectHashTable.containsKey(objectType)) {
			int i = apiObjectHashTable.get(objectType);
			i++;
			apiObjectHashTable.put(objectType, i);
			return i;
		} else {
			apiObjectHashTable.put(objectType, 0);
		}
		return 0;
	}

	public static void saveTraceObjectVectorToFile(Vector traceObjectVector,
			String filename) {
		ObjectIO.saveToFile(filename, traceObjectVector);
	}

	/**
	 * Save Trace Object Vector to given filename
	 *
	 * @param filename
	 */
	public void saveToFile(String filename) {
		saveTraceObjectVectorToFile(traceObjectVector, filename);
	}

}
